@@ -1,5 +1,5 @@
 /* # skkinput (Simple Kana-Kanji Input)
- * OverWin.c --- Making OverTheSpotWinWidget.
+ * OnWin.c --- Making OnTheSpotWinWidget.
  * This file is part of skkinput.
  * Copyright (C) 2002
  * Takashi SAKAMOTO (sakamoto@yajima.kuis.kyoto-u.ac.jp)
@@ -65,7 +65,7 @@ static XtResource OnWin_resources[] = {
 	/* Minibuffer ȿžɽ뤫ɤȤäˤʤ롣*/
 	{ XtNreverseVideo,	XtCReverseVideo,	XtRBoolean,	sizeof (Boolean),
 	  offset(reverse_video),	XtRImmediate,	(XtPointer) FALSE},
-	/* Overthespot Ѥեȵڤӥ޻եȡ*/
+	/* Onthespot Ѥեȵڤӥ޻եȡ*/
 	{ XtNfontset,		XtCFontset,	XtRString,	sizeof (String),
 	  offset(fontset_string),	XtRImmediate,	(XtPointer)DEFAULT_FONTSET },
 	/* Minibuffer Window 褹ΤѤڤӥ޻եȡ*/
@@ -1315,6 +1315,7 @@ OnWin_fixcallback (
 
 	fClear	= wgThis->onthespotWin.m_fPreeditActive ;
 	if (fClear) {
+ 		arg.m_nCaret		= 0 ;
 		arg.m_nChgFirst		= 0 ;
 		arg.m_nChgLength	= myCharStrlen (wgThis->onthespotWin.m_strPreeditText) ;
 		arg.m_pText			= NULL ;
@@ -1322,6 +1323,8 @@ OnWin_fixcallback (
 		arg.m_pAttribute	= NULL ;
 		arg.m_nAttribute	= 0 ;
 		XtCallCallbacks (gw, XtNpreeditDrawCallback, &arg) ;
+ 		MYCHAR_SET_END_OF_STRING (wgThis->onthespotWin.m_strPreeditText [0]) ;
+ 		wgThis->onthespotWin.m_nPreeditCaret	= 0 ;
 	}
 	XtCallCallbacks (gw, XtNtextNotify, caller) ;
 
