@@ -84,6 +84,8 @@ static XtResource OTSW_resources[] = {
   /* key event  client ֤Ѥ callback */
   { XtNkeybackNotify, XtCCallback, XtRCallback, sizeof (caddr_t), 
     offset(keybackcallback), XtRCallback, (caddr_t)NULL },
+  { XtNtextNotify,	XtCCallback, XtRCallback, sizeof (XtCallbackList),
+	offset (m_lstTextCallback),	XtRCallback, (caddr_t) NULL },
   /* egg ߴ j-newline ݤ*/
   { XtNeggLikeNewline, XtCEggLikeNewline, XtRImmediate, sizeof (Boolean),
     XtOffsetOf (OverthespotWinRec, overthespotWin.buffer.egg_like_newline),
@@ -102,6 +104,8 @@ static XtResource OTSW_resources[] = {
   /* ߥ˥Хåեξõ롣*/
   { XtNclearMinibuffer, XtCClearMinibuffer, XtRImmediate, sizeof (Boolean),
     offset (clearMinibuffer), XtRImmediate, (XtPointer)FALSE },
+  {	XtNinput,	XtCInput,	XtRBoolean,		sizeof (Boolean),
+    offset (m_fInputFocus),	XtRImmediate,  (XtPointer)TRUE },
   /* ¦ɽġ*/
   { XtNsouthCursor, XtCSouthCursor, XtRBoolean, sizeof (Boolean),
     offset (south_cursor), XtRImmediate,  (XtPointer)FALSE },
@@ -208,6 +212,7 @@ static void OTSW_wmShellDestroyCallback
  (Widget gw, caddr_t client, caddr_t caller) ;
 static void Canvas_DestroyCallback
  (Widget gw, caddr_t client, caddr_t caller) ;
+static void		OverWin_fixcallback		(Widget, XtPointer, XtPointer) ;
 
 /* common style */
 extern void skkinput_bufferInit (struct skkinputBuffer *buffer) ;
@@ -641,13 +646,15 @@ static void OTSW_RealizeMinibuffer (Widget gw)
 		font_height = w->overthespotWin.mfont_height ;
 	}
 	minibuffer_popup = XtVaCreatePopupShell
-		("minibuffer", wmcloseShellWidgetClass, gw, XtNinput, TRUE, 
-		 XtNmappedWhenManaged, TRUE, 
-		 XtNwidth, winwidth, XtNheight, winheight, 
-		 XtNcolormap, gw->core.colormap,
-		 XtNreverseVideo, w->overthespotWin.reverse_video,
-		 XtNbackground, w->overthespotWin.background,
-		 XtNforeground, w->overthespotWin.foreground, NULL) ;
+		("minibuffer", wmcloseShellWidgetClass, gw,
+		 XtNinput, 				w->overthespotWin.m_fInputFocus,
+		 XtNmappedWhenManaged,	TRUE, 
+		 XtNwidth,				winwidth,
+		 XtNheight,				winheight, 
+		 XtNcolormap,			gw->core.colormap,
+		 XtNreverseVideo,		w->overthespotWin.reverse_video,
+		 XtNbackground,			w->overthespotWin.background,
+		 XtNforeground,			w->overthespotWin.foreground, NULL) ;
 	XtAddCallback
 		(minibuffer_popup, XtNwmcloseNotify, 
 		 (XtCallbackProc)OTSW_wmMessageCloseCallback, gw) ;
@@ -709,10 +716,11 @@ static void OTSW_RealizeMinibuffer (Widget gw)
 	return ;
 }
 
-static void OTSW_CreateMinibuffer
- (Widget gw)
+static void
+OTSW_CreateMinibuffer (
+	register Widget		gw)
 {
-	OverthespotWinWidget w =  (OverthespotWinWidget)gw ;
+	register OverthespotWinWidget w =  (OverthespotWinWidget)gw ;
 
 	if ((w->overthespotWin.attribute_mask & CAStatusArea) &&
 		w->overthespotWin.client_window != w->overthespotWin.focus_window){
@@ -728,27 +736,29 @@ static void OTSW_CreateMinibuffer
 		}
 		/* Ѵ饤ȤŽդפΥߥ˥ХåեǤϤȤƤ礭
 		 * ­ʤΤǡ˳פˤ롣*/
-		XDestroyWindow
-			(XtDisplay (gw), w->overthespotWin.minibuffer_win) ;
+		XDestroyWindow (XtDisplay (gw), w->overthespotWin.minibuffer_win) ;
 	}
 	w->overthespotWin.minibuffer_win = None ;
 	w->overthespotWin.minibuffer_ofstx = 0 ;
 	OTSW_RealizeMinibuffer (gw) ;
-	
-	XtSetKeyboardFocus (w->overthespotWin.minibuffer_popup, gw) ;
-	XtSetKeyboardFocus (w->overthespotWin.minibuffer_canvas, gw) ;
+
+	if (w->overthespotWin.m_fInputFocus) {
+		XtSetKeyboardFocus (w->overthespotWin.minibuffer_popup, gw) ;
+		XtSetKeyboardFocus (w->overthespotWin.minibuffer_canvas, gw) ;
+	}
 	return ;
 }
 
 /*
  * Widget β˸ƤФؿ
  */
-static void OTSW_Realize
- (Widget gw, XtValueMask *valueMask, XSetWindowAttributes *attrs)
+static void OTSW_Realize (
+	register Widget					gw,
+	register XtValueMask*			valueMask,
+	register XSetWindowAttributes*	attrs)
 {
-	OverthespotWinWidget w =  (OverthespotWinWidget)gw ;
-	CoreWidgetClass super =
-		(CoreWidgetClass)XtClass (gw)->core_class.superclass ;
+	register OverthespotWinWidget	w		= (OverthespotWinWidget)gw ;
+	register CoreWidgetClass		super	= (CoreWidgetClass)XtClass (gw)->core_class.superclass ;
 
 	/* ŬڤʥɥѰդƤ*/
 	(*super->core_class.realize) (gw, valueMask, attrs) ;
@@ -759,6 +769,9 @@ static void OTSW_Realize
 		XtDestroyWidget (gw) ;
 		return ;
 	}
+	if (XtHasCallbacks (gw, XtNfixNotify) != XtCallbackHasSome) 
+		XtAddCallback (gw, XtNfixNotify, OverWin_fixcallback, NULL) ;
+
 	if (w->overthespotWin.focus_window == None){
 		w->overthespotWin.focus_window = w->overthespotWin.client_window ;
 		if (! (w->overthespotWin.attribute_mask & CAClientArea))
@@ -796,7 +809,7 @@ static void OTSW_Realize
 	 * ΰ֤ʤ⡼ɥƤʤ
 	 */
 	OTSW_resetupModeWindow (gw) ;
-	OTSW_ReconfigureColor (gw, CAColor | CAColormap) ;
+	OTSW_ReconfigureColor (gw, CAForegroundPixel | CABackgroundPixel | CAColormap) ;
 #if 0
 	XSync (XtDisplay (gw), False) ;
 #endif
@@ -846,65 +859,65 @@ static void OTSW_IgnoreAllWindowEvent (Widget gw)
  */
 static void OTSW_Destroy (Widget gw)
 {
-  OverthespotWinWidget w =  (OverthespotWinWidget)gw ;
-  struct SKKInputNode *node, *pNode ;
-  /*
-   * XFreeFont (XtDisplay (gw), w->overthespotWin.fs_kanji) ;
-   * XFreeFont (XtDisplay (gw), w->overthespotWin.fs_roman) ;
-   * ɤ顢Widget ˳ݤƤ櫓ʤߤʤΤǡ
-   * ȤϤޤߤ
-   */
-  /* Destroy ʾ塢ʲΥɥΥ٥Ȥ̵뤹롣*/
-  OTSW_IgnoreAllWindowEvent (gw) ;
-  OTSW_ReleaseAllGCs (gw) ;
-  OTSW_ReleaseMinibufferGCs (gw) ;
-  OTSW_FreeFontSet (gw) ;
-
-  if (w->overthespotWin.modeshell_win != None)
-    XDestroyWindow (XtDisplay (gw), w->overthespotWin.modeshell_win) ;
-  if (w->overthespotWin.topline_win != None)
-    XDestroyWindow (XtDisplay (gw), w->overthespotWin.topline_win) ;
-  if (w->overthespotWin.midline_win != None)
-    XDestroyWindow (XtDisplay (gw), w->overthespotWin.midline_win) ;
-  if (w->overthespotWin.botline_win != None)
-    XDestroyWindow (XtDisplay (gw), w->overthespotWin.botline_win) ;
-  if (w->overthespotWin.minibuffer_win != None)
-    XDestroyWindow (XtDisplay (gw), w->overthespotWin.minibuffer_win) ;
+	OverthespotWinWidget w =  (OverthespotWinWidget)gw ;
+	struct SKKInputNode *node, *pNode ;
+	/*
+	 * XFreeFont (XtDisplay (gw), w->overthespotWin.fs_kanji) ;
+	 * XFreeFont (XtDisplay (gw), w->overthespotWin.fs_roman) ;
+	 * ɤ顢Widget ˳ݤƤ櫓ʤߤʤΤǡ
+	 * ȤϤޤߤ
+	 */
+	/* Destroy ʾ塢ʲΥɥΥ٥Ȥ̵뤹롣*/
+	OTSW_IgnoreAllWindowEvent (gw) ;
+	OTSW_ReleaseAllGCs (gw) ;
+	OTSW_ReleaseMinibufferGCs (gw) ;
+	OTSW_FreeFontSet (gw) ;
+	
+	if (w->overthespotWin.modeshell_win != None)
+		XDestroyWindow (XtDisplay (gw), w->overthespotWin.modeshell_win) ;
+	if (w->overthespotWin.topline_win != None)
+		XDestroyWindow (XtDisplay (gw), w->overthespotWin.topline_win) ;
+	if (w->overthespotWin.midline_win != None)
+		XDestroyWindow (XtDisplay (gw), w->overthespotWin.midline_win) ;
+	if (w->overthespotWin.botline_win != None)
+		XDestroyWindow (XtDisplay (gw), w->overthespotWin.botline_win) ;
+	if (w->overthespotWin.minibuffer_win != None)
+		XDestroyWindow (XtDisplay (gw), w->overthespotWin.minibuffer_win) ;
     
-  /*
-   * ҥȥƤؤ֤
-   */
-  if (w->overthespotWin.historyAttribute != NULL){
-    /* ҥȥˤĤƿƤ֤ޤ*/
-    w->overthespotWin.historyAttribute->history_start =
-      w->overthespotWin.buffer.hist_start ;
-    w->overthespotWin.historyAttribute->history_end =
-      w->overthespotWin.buffer.hist_end ;
-    /* ξѴ⡼ɤξˤĤƿƤؤ֤*/
-    w->overthespotWin.historyAttribute->overthespot_like_input =
-      w->overthespotWin.buffer.overthespot_like_input ;
-    /* ޤĤβ̾⡼ɤξƤؤ֤ޤ*/
-    if (w->overthespotWin.buffer.topbuffer != NULL){
-      w->overthespotWin.historyAttribute->j_mode =
-	w->overthespotWin.buffer.topbuffer->j_mode ;
-      w->overthespotWin.historyAttribute->j_zenkaku =
-	w->overthespotWin.buffer.topbuffer->j_zenkaku ;
-      w->overthespotWin.historyAttribute->j_katakana_mode =
-	w->overthespotWin.buffer.topbuffer->j_katakana_mode ;
-    }
-  }
-  /*
-   * ХåեƲƤ
-   */
-  node =  (w->overthespotWin.buffer).lastbuffer ;
-  while (node != NULL){
-    pNode = node->parentbuffer ;
-    free_Minibuffer (gw, & (w->overthespotWin.buffer), node) ;
-    node  = pNode ;
-  }
-  XtCallCallbacks (gw, XtNendNotify, w->overthespotWin.historyAttribute) ;
-  w->overthespotWin.historyAttribute = NULL ;
-  return ;
+	/*
+	 * ҥȥƤؤ֤
+	 */
+	if (w->overthespotWin.historyAttribute != NULL){
+		/* ҥȥˤĤƿƤ֤ޤ*/
+		w->overthespotWin.historyAttribute->history_start =
+			w->overthespotWin.buffer.hist_start ;
+		w->overthespotWin.historyAttribute->history_end =
+			w->overthespotWin.buffer.hist_end ;
+		/* ξѴ⡼ɤξˤĤƿƤؤ֤*/
+		w->overthespotWin.historyAttribute->overthespot_like_input =
+			w->overthespotWin.buffer.overthespot_like_input ;
+		/* ޤĤβ̾⡼ɤξƤؤ֤ޤ*/
+		if (w->overthespotWin.buffer.topbuffer != NULL){
+			w->overthespotWin.historyAttribute->j_mode =
+				w->overthespotWin.buffer.topbuffer->j_mode ;
+			w->overthespotWin.historyAttribute->j_zenkaku =
+				w->overthespotWin.buffer.topbuffer->j_zenkaku ;
+			w->overthespotWin.historyAttribute->j_katakana_mode =
+				w->overthespotWin.buffer.topbuffer->j_katakana_mode ;
+		}
+	}
+	/*
+	 * ХåեƲƤ
+	 */
+	node =  (w->overthespotWin.buffer).lastbuffer ;
+	while (node != NULL){
+		pNode = node->parentbuffer ;
+		free_Minibuffer (gw, & (w->overthespotWin.buffer), node) ;
+		node  = pNode ;
+	}
+	XtCallCallbacks (gw, XtNendNotify, w->overthespotWin.historyAttribute) ;
+	w->overthespotWin.historyAttribute = NULL ;
+	return ;
 }
 
 /*
@@ -1028,51 +1041,51 @@ static void OTSW_ResizeMinibufferAndModeshell (Widget gw)
  */
 static void OTSW_ReconfigureColor (Widget gw, unsigned long valuemask)
 {
-  OverthespotWinWidget w =  (OverthespotWinWidget)gw ;
-  unsigned long mask ;
-  XSetWindowAttributes xswa ;
+	OverthespotWinWidget w =  (OverthespotWinWidget)gw ;
+	unsigned long mask ;
+	XSetWindowAttributes xswa ;
 
-  mask = 0L ;
-  if (valuemask & CAColor){
-    mask |=  (CWBackPixel | CWBorderPixel) ;
-    xswa.background_pixel = w->overthespotWin.obackground ;
-    xswa.border_pixel     = w->overthespotWin.oforeground ;
-  }
-  if (valuemask & CAColormap){
-    mask |= CWColormap ;
-    xswa.colormap = w->overthespotWin.colormap ;
-  }
-  if (mask){
-    /*   ֤󡣤ʤꤷʤȤʤΤġ *
-     * ե꡼֥եȤ衢եȡ                     */
-    XChangeWindowAttributes
-       (XtDisplay (gw), w->overthespotWin.modeshell_win, 
-	mask, &xswa) ;
-    XChangeWindowAttributes
-       (XtDisplay (gw), w->overthespotWin.topline_win, 
-	mask, &xswa) ;
-    XChangeWindowAttributes
-       (XtDisplay (gw), w->overthespotWin.midline_win, 
-	mask, &xswa) ;
-    XChangeWindowAttributes
-       (XtDisplay (gw), w->overthespotWin.botline_win, 
-	mask, &xswa) ;
-    /* ⤷饤ŽդפΥߥ˥Хåեʤ顢⹹*/
-    if (w->overthespotWin.minibuffer_win != None){
-      XChangeWindowAttributes
-	 (XtDisplay (gw), w->overthespotWin.minibuffer_win, 
-	  mask, &xswa) ;
-    }
-  }
-  if (! (valuemask & CAColor))
-    return ;
+	mask = 0L ;
+	if (valuemask & CABackgroundPixel){
+		mask	|= CWBackPixel ;
+		xswa.background_pixel = w->overthespotWin.obackground ;
+	}
+	if (valuemask & CAForegroundPixel) {
+		mask	|= CWBorderPixel ;
+		xswa.border_pixel     = w->overthespotWin.oforeground ;
+	}
+	if (valuemask & CAColormap){
+		mask |= CWColormap ;
+		xswa.colormap = w->overthespotWin.colormap ;
+	}
+	if (mask){
+		XChangeWindowAttributes
+			(XtDisplay (gw), w->overthespotWin.modeshell_win, 
+			 mask, &xswa) ;
+		XChangeWindowAttributes
+			(XtDisplay (gw), w->overthespotWin.topline_win, 
+			 mask, &xswa) ;
+		XChangeWindowAttributes
+			(XtDisplay (gw), w->overthespotWin.midline_win, 
+			 mask, &xswa) ;
+		XChangeWindowAttributes
+			(XtDisplay (gw), w->overthespotWin.botline_win, 
+			 mask, &xswa) ;
+		if (w->overthespotWin.minibuffer_win != None){
+			XChangeWindowAttributes
+				(XtDisplay (gw), w->overthespotWin.minibuffer_win, 
+				 mask, &xswa) ;
+		}
+	}
+	if (! (valuemask & (CAForegroundPixel | CABackgroundPixel)))
+		return ;
 
-  /* GC  foreground, background ѹ롣*/
-  if (XtIsRealized (gw)){
-    OTSW_ReleaseAllGCs (gw) ;
-    OTSW_CreateAllGCs (gw) ;
-  }
-  return ;
+	/* GC  foreground, background ѹ롣*/
+	if (XtIsRealized (gw)){
+		OTSW_ReleaseAllGCs (gw) ;
+		OTSW_CreateAllGCs (gw) ;
+	}
+	return ;
 }
 
 /*
@@ -1101,9 +1114,6 @@ static Boolean OTSW_SetValues
   /* Ѵ°ν׵᤬褿ȸդꡣ*/
   if (reqw->overthespotWin.camsg != NULL){
     int ret ;
-    /* Ѵ°ˤǸ­Τɤ *
-     * äƤߤäʤ͡                                     *
-     * ա֤Ǥ͡äƤߤޤ礦Ĥ󡪡             */
     camsg = reqw->overthespotWin.camsg ;
     reqw->overthespotWin.camsg = neww->overthespotWin.camsg = 
       curw->overthespotWin.camsg = NULL ;
@@ -1200,38 +1210,22 @@ static int OTSW_ReconfigureFocusPosition
 }
 
 /*
- * ʤ굯ưƤʳǥեưȤ̵
- * ꤤ(̿δְ㤤ʤΤ)򲿤Ȥؿ 
- *----
- * Netscape ׵ФƤΤǡޤ
- * οͤˤȤäƤϡNetscape ơNetscape ܸ첽ޤơBus 
- * Error С kinput2  skkinput ΤʤΡ
- * ƤȤ顣錄ᤷǤʾС
  */
-static void OTSW_ChangeFocusWindow
- (Widget gw, Window focus_win)
+static void OTSW_ChangeFocusWindow (
+	register Widget	gw,
+	register Window	focus_win)
 {
+	register Display*	pDisplay	= XtDisplay (gw) ;
 	OverthespotWinWidget w =  (OverthespotWinWidget)gw ;
 
-	XDestroyWindow (XtDisplay (gw), w->overthespotWin.topline_win) ;
-	XDestroyWindow (XtDisplay (gw), w->overthespotWin.midline_win) ;
-	XDestroyWindow (XtDisplay (gw), w->overthespotWin.botline_win) ;
-
-	w->overthespotWin.topline_win = OTSW_CreateSubCanvas
-		(gw, focus_win, &w->overthespotWin.topline_probe, 0) ;
-	w->overthespotWin.midline_win = OTSW_CreateSubCanvas
-		(gw, focus_win, &w->overthespotWin.midline_probe, 0) ;
-	w->overthespotWin.botline_win = OTSW_CreateSubCanvas
-		(gw, focus_win, &w->overthespotWin.botline_probe, 0) ;
+	XReparentWindow (pDisplay, w->overthespotWin.topline_win, focus_win, 0, 0) ;
+	XReparentWindow (pDisplay, w->overthespotWin.midline_win, focus_win, 0, 0) ;
+	XReparentWindow (pDisplay, w->overthespotWin.botline_win, focus_win, 0, 0) ;
 	
 	/* focus window ˤϤĤƤɥ˴롣*/
-	if (!w->overthespotWin.fixed_modeshell){
-		XDestroyWindow (XtDisplay (gw), w->overthespotWin.modeshell_win) ;
-		
-		w->overthespotWin.modeshell_win = OTSW_CreateSubCanvas
-			(gw, focus_win, &w->overthespotWin.modeshell_probe, 
-			 w->overthespotWin.modeshell_border_width) ;
-	}
+	if (!w->overthespotWin.fixed_modeshell)
+		XReparentWindow (pDisplay, w->overthespotWin.modeshell_win, focus_win, 0, 0) ;
+
 	w->overthespotWin.focus_window = focus_win ;
 	OTSW_ReleaseAllGCs (gw) ;
 	OTSW_CreateAllGCs (gw) ;
@@ -1244,172 +1238,166 @@ static void OTSW_ChangeFocusWindow
 static int OTSW_ChangeAttributes
  (Widget gw, unsigned long valuemask, struct ConvAttrs *values)
 {
-  OverthespotWinWidget w   =  (OverthespotWinWidget)gw ;
-  int ret = False, rec_col ;
+	OverthespotWinWidget w   =  (OverthespotWinWidget)gw ;
+	int ret = False, rec_col ;
 
-  if (valuemask & CAFocusWindow){
+	if (valuemask & CAFocusWindow){
+#if defined (DEBUG)
+		fprintf (stderr, "(OTSW)Focus window (%ld)\n", values->focus_window) ;
+#endif
+		/* եäƼºߤΡ Ȥ¤Ϥ⤦ä *
+		 * ɬפȻפ*/
+		if (values->focus_window != None){
+			/* ꤷȦΥեʪˤʤäƤ顢̣Ǥ */
+			/* ʤ ȤäǤ*/
+			if (w->overthespotWin.focus_window != None &&
+				w->overthespotWin.focus_window != values->focus_window){
+				/* ե롣*/
+				OTSW_ChangeFocusWindow
+					(gw, values->focus_window) ;
+				/* Υ饤ȥꥢФϺˤƤ
+				 * ʤʤΤǡ˴롣*/
+				w->overthespotWin.attribute_mask &= ~ (CAClientArea) ;
+			} else {
+				w->overthespotWin.focus_window = values->focus_window ;
+			}
+			/* Ѵ饤ȤΰФ̵ä鼫Ѱդ *
+			 * 줬ɤʤݾڤϤʤΤɡѴ *
+			 * Ȥʤ̵ */
+			if (!((w->overthespotWin.attribute_mask) & CAClientArea)){
+				XGetRectangleOfWindow
+					(XtDisplay (gw), w->overthespotWin.focus_window,
+					 &w->overthespotWin.client_area) ;
+			}
+			ret = True ;
+		}
+	}
+	if (valuemask & CAStatusArea){
+		/* ɸνä⤷ɽΰ褬Ƥ硣*/
+		if (values->status_area.x != w->overthespotWin.status_area.x ||
+			values->status_area.y != w->overthespotWin.status_area.y ||
+			values->status_area.width  != w->overthespotWin.status_area.width || 
+			values->status_area.height != w->overthespotWin.status_area.height){
+			w->overthespotWin.status_area.x = values->status_area.x ;
+			w->overthespotWin.status_area.y = values->status_area.y ;
+			w->overthespotWin.status_area.width  = values->status_area.width ;
+			w->overthespotWin.status_area.height = values->status_area.height ;
+			w->overthespotWin.attribute_mask |= CAStatusArea ;
 #ifdef DEBUG
-    fprintf
-       (stderr, "(OTSW)Focus window (%ld)\n", values->focus_window) ;
+			printf ("Status Area: (x,y,w,h) = (%d,%d,%d,%d)\n",
+					values->status_area.x, values->status_area.y,
+					values->status_area.width, values->status_area.height) ;
 #endif
-    /* եäƼºߤΡ Ȥ¤Ϥ⤦ä *
-     * ɬפȻפ*/
-    if (values->focus_window != None){
-      /* ꤷȦΥեʪˤʤäƤ顢̣Ǥ */
-      /* ʤ ȤäǤ*/
-      if (w->overthespotWin.focus_window != None &&
-	  w->overthespotWin.focus_window != values->focus_window){
-	/* ե롣*/
-	OTSW_ChangeFocusWindow
-	   (gw, values->focus_window) ;
-	/* Υ饤ȥꥢФϺˤƤ
-	 * ʤʤΤǡ˴롣*/
-	w->overthespotWin.attribute_mask &= ~ (CAClientArea) ;
-#if 0
-	/* Netscape ǤϤƤ */
-	XtDestroyWidget (gw) ;
-	return False ;
-#endif
-      } else {
-	w->overthespotWin.focus_window = values->focus_window ;
-      }
-      /* Ѵ饤ȤΰФ̵ä鼫Ѱդ *
-       * 줬ɤʤݾڤϤʤΤɡѴ *
-       * Ȥʤ̵ */
-      if (!((w->overthespotWin.attribute_mask) & CAClientArea)){
-	XGetRectangleOfWindow
-	   (XtDisplay (gw), w->overthespotWin.focus_window,
-	    &w->overthespotWin.client_area) ;
-      }
-      ret = True ;
-    }
-  }
-  if (valuemask & CAStatusArea){
-    /* ɸνä⤷ɽΰ褬Ƥ硣*/
-    if (values->status_area.x != w->overthespotWin.status_area.x ||
-        values->status_area.y != w->overthespotWin.status_area.y ||
-        values->status_area.width  != w->overthespotWin.status_area.width || 
-        values->status_area.height != w->overthespotWin.status_area.height){
-      w->overthespotWin.status_area.x = values->status_area.x ;
-      w->overthespotWin.status_area.y = values->status_area.y ;
-      w->overthespotWin.status_area.width  = values->status_area.width ;
-      w->overthespotWin.status_area.height = values->status_area.height ;
-      w->overthespotWin.attribute_mask |= CAStatusArea ;
-#ifdef DEBUG
-      printf ("Status Area: (x,y,w,h) = (%d,%d,%d,%d)\n",
-	      values->status_area.x, values->status_area.y,
-	      values->status_area.width, values->status_area.height) ;
-#endif
-      ret = True ;
-    }
-  }
-  rec_col = False ;
-  /* 顼ޥåפѹ׵᤬Ƥ顢롣*/
-  if ((valuemask & CAColormap) &&
-     w->overthespotWin.colormap != values->colormap){
-    w->overthespotWin.colormap = values->colormap ;
+			ret = True ;
+		}
+	}
+	rec_col = False ;
+	/* 顼ޥåפѹ׵᤬Ƥ顢롣*/
+	if ((valuemask & CAColormap) &&
+		w->overthespotWin.colormap != values->colormap){
+		w->overthespotWin.colormap = values->colormap ;
 #if defined(DEBUG)
-    fprintf
-       (stderr, "(OTSW)Change colormap.\n") ;
+		fprintf (stderr, "(OTSW)Change colormap.\n") ;
 #endif
-    rec_col = True ;
-  }
-  /* ե饦ɥ顼ڤӥХå饦ɥ顼ѹ׵᤬ *
-   * 顢ġ*/
-  if ((valuemask & CAColor) &&
-      (w->overthespotWin.obackground != values->background ||
-       w->overthespotWin.oforeground != values->foreground)){
+		rec_col = True ;
+	}
+	/* ե饦ɥ顼ڤӥХå饦ɥ顼ѹ׵᤬ *
+	 * 顢ġ*/
+	if ((valuemask & CAForegroundPixel) &&
+		w->overthespotWin.oforeground != values->foreground){
 #if defined(DEBUG)
-    fprintf
-       (stderr, "(OTSW)Change foreground and background colors.\n") ;
+		fprintf (stderr, "(OTSW)Change foreground and background colors.\n") ;
 #endif
-    w->overthespotWin.obackground = values->background ;
-    w->overthespotWin.oforeground = values->foreground ;
-    rec_col = True ;
-  }
-  if (XtIsRealized (gw) && rec_col){
-    OTSW_ReconfigureColor (gw, valuemask) ;
-    ret = True ;
-  }
-  /* Ѵ饤ȤɽΰνΤ褿ν*/
-  if (valuemask & CAClientArea){
-    /* ɸνä⤷ɽΰ褬Ƥ硣*/
-    if (values->client_area.x != w->overthespotWin.client_area.x ||
-        values->client_area.y != w->overthespotWin.client_area.y ||
-        values->client_area.width != w->overthespotWin.client_area.width || 
-        values->client_area.height != w->overthespotWin.client_area.height){
-      /* Ѵ饤ȤɽΰѹФƤ*/
-      w->overthespotWin.client_area    = values->client_area ;
+		w->overthespotWin.oforeground = values->foreground ;
+		rec_col = True ;
+	}
+	if ((valuemask & CABackgroundPixel) &&
+		w->overthespotWin.obackground != values->background) {
+		w->overthespotWin.obackground = values->background ;
+		rec_col = True ;
+	}
+	if (XtIsRealized (gw) && rec_col){
+		OTSW_ReconfigureColor (gw, valuemask) ;
+		ret = True ;
+	}
+	/* Ѵ饤ȤɽΰνΤ褿ν*/
+	if (valuemask & CAClientArea){
+		/* ɸνä⤷ɽΰ褬Ƥ硣*/
+		if (values->client_area.x != w->overthespotWin.client_area.x ||
+			values->client_area.y != w->overthespotWin.client_area.y ||
+			values->client_area.width != w->overthespotWin.client_area.width || 
+			values->client_area.height != w->overthespotWin.client_area.height){
+			/* Ѵ饤ȤɽΰѹФƤ*/
+			w->overthespotWin.client_area    = values->client_area ;
 #ifdef DEBUG
-      fprintf
-	 (stderr, "(OTSW)Client pos:(%d,%d), size:%d x %d.\n",
-	  w->overthespotWin.client_area.x,
-	  w->overthespotWin.client_area.y,
-	  w->overthespotWin.client_area.width,
-	  w->overthespotWin.client_area.height) ;
+			fprintf	(stderr, "(OTSW)Client pos:(%d,%d), size:%d x %d.\n",
+					 w->overthespotWin.client_area.x,
+					 w->overthespotWin.client_area.y,
+					 w->overthespotWin.client_area.width,
+					 w->overthespotWin.client_area.height) ;
 #endif
-      w->overthespotWin.attribute_mask |= CAClientArea ;
-      ret = True ;
-    }
-  }
-  /* ɸȤ׵᤬ƤνԤ*/
-  if (valuemask & CASpotLocation){
-    if (w->overthespotWin.focus_window != None){
-      int prev_focus ;
+			w->overthespotWin.attribute_mask |= CAClientArea ;
+			ret = True ;
+		}
+	}
+	/* ɸȤ׵᤬ƤνԤ*/
+	if (valuemask & CASpotLocation){
+		if (w->overthespotWin.focus_window != None){
+			int prev_focus ;
 #if defined(DEBUG)
-      fprintf (stderr, "CASPOT nanode FOCUSWINDOW miru (%ld).\n",
-	      w->overthespotWin.focus_window) ;
+			fprintf (stderr, "CASPOT nanode FOCUSWINDOW miru (%ld).\n",
+					 w->overthespotWin.focus_window) ;
 #endif
-      prev_focus = w->overthespotWin.is_focus ;
-      w->overthespotWin.is_focus = focusCheck (gw) ;
-      if (prev_focus != w->overthespotWin.is_focus){
-	ret = True ;
-      }
-      if (OTSW_ReconfigureFocusPosition (gw))
-	ret = True ;
-    }
-    if (values->spot_x != w->overthespotWin.spot_x ||
-	values->spot_y != w->overthespotWin.spot_y){
+			prev_focus = w->overthespotWin.is_focus ;
+			w->overthespotWin.is_focus = focusCheck (gw) ;
+			if (prev_focus != w->overthespotWin.is_focus){
+				ret = True ;
+			}
+			if (OTSW_ReconfigureFocusPosition (gw))
+				ret = True ;
+		}
+		if (values->spot_x != w->overthespotWin.spot_x ||
+			values->spot_y != w->overthespotWin.spot_y){
 #ifdef DEBUG
-      fprintf (stderr, "Get Basho Shitei meirei.\n") ;
+			fprintf (stderr, "Get Basho Shitei meirei.\n") ;
 #endif
-      /* ֤뤵*/
-      w->overthespotWin.spot_x = values->spot_x ;
-      w->overthespotWin.spot_y = values->spot_y ;
-      ret = True ;
-    }
-  }
-  /* եȾνäν*/
-  if (valuemask & CAFonts){
-    /* եȾ򹹿롣λ饤¦ΥեȤξ
-     * ƹ碌ΤȤ롣㤨饤ȤեȤ
-     * ꤷƤʤƥǥեȤΥեȤȤȤˤʤꡢΥե
-     * Ȥι⤵ʤƤġ(ɤΤ) */
-    if (fontMgr_GetFontSetInfo
-	 (values->fontset,
-	  &w->overthespotWin.font_height,
-	  &w->overthespotWin.font_ascent)){
-      /* եȥåȤ򥳥ԡ롣*/
-      fontMgr_CopyFontSet
-	 (XtDisplay (gw), w->overthespotWin.fontset, values->fontset) ;
-      /* ߥ˥Хåեȥ⡼ɥ礭롣*/
-      if (XtIsRealized (gw))
-	OTSW_ResizeMinibufferAndModeshell (gw) ;
-      ret = True ;
-    }
-  }
-  /* եȤι⤵˴ؤƤˤϡġ*/
-  if (valuemask & CALineSpacing){
-    if (values->linespacing > 0){
-      w->overthespotWin.font_height = values->linespacing ;
-      ret = True ;
-    }
-  }
-  if (( (valuemask & CAFonts) ||  (valuemask & CAStatusArea)) &&
-      w->overthespotWin.minibuffer_win != None){
-    OTSW_ConfigureSubWindowMinibuffer (gw, True) ;
-  }
-  return ret ;
+			/* ֤뤵*/
+			w->overthespotWin.spot_x = values->spot_x ;
+			w->overthespotWin.spot_y = values->spot_y ;
+			ret = True ;
+		}
+	}
+	/* եȾνäν*/
+	if (valuemask & CAFonts){
+		/* եȾ򹹿롣λ饤¦ΥեȤξ
+		 * ƹ碌ΤȤ롣㤨饤ȤեȤ
+		 * ꤷƤʤƥǥեȤΥեȤȤȤˤʤꡢΥե
+		 * Ȥι⤵ʤƤġ(ɤΤ) */
+		if (fontMgr_GetFontSetInfo
+			(values->fontset,
+			 &w->overthespotWin.font_height,
+			 &w->overthespotWin.font_ascent)){
+			/* եȥåȤ򥳥ԡ롣*/
+			fontMgr_CopyFontSet
+				(XtDisplay (gw), w->overthespotWin.fontset, values->fontset) ;
+			/* ߥ˥Хåեȥ⡼ɥ礭롣*/
+			if (XtIsRealized (gw))
+				OTSW_ResizeMinibufferAndModeshell (gw) ;
+			ret = True ;
+		}
+	}
+	/* եȤι⤵˴ؤƤˤϡġ*/
+	if (valuemask & CALineSpacing){
+		if (values->linespacing > 0){
+			w->overthespotWin.font_height = values->linespacing ;
+			ret = True ;
+		}
+	}
+	if (( (valuemask & CAFonts) ||  (valuemask & CAStatusArea)) &&
+		w->overthespotWin.minibuffer_win != None){
+		OTSW_ConfigureSubWindowMinibuffer (gw, True) ;
+	}
+	return ret ;
 }
 
 /*
@@ -2610,3 +2598,13 @@ static void Canvas_DestroyCallback
   return ;
 }
 
+static void
+OverWin_fixcallback (
+	register Widget		gw,
+	register XtPointer	client,
+	register XtPointer	caller)
+{
+	XtCallCallbacks (gw, XtNtextNotify, caller) ;
+	return ;
+}
+
