@@ -74,6 +74,8 @@ static XtResource resources[] = {
   /* key event  client ֤Ѥ callback */
   { XtNkeybackNotify, XtCCallback, XtRCallback, sizeof(caddr_t), 
     offset(keybackcallback), XtRCallback, (caddr_t)NULL },
+  { XtNtextNotify,	XtCCallback, XtRCallback, sizeof (XtCallbackList),
+	offset (m_lstTextCallback),	XtRCallback, (caddr_t) NULL },
   /* Shift-Space  skkinput Ĥ褦ˤ뤫ݤ*/
   { XtNcompatibleCloseSkkinputKey, XtCCompatibleCloseSkkinputKey,
     XtRBoolean, sizeof ( Boolean ),
@@ -119,6 +121,8 @@ static XtResource resources[] = {
     sizeof( Boolean ), 
     XtOffsetOf( SeparateWinRec, separateWin.buffer.toggleControlMode ),
     XtRImmediate, ( XtPointer )FALSE },
+  {	XtNinput,	XtCInput,	XtRBoolean,	sizeof (Boolean),
+	offset (m_fInputFocus), XtRImmediate, (XtPointer) TRUE, },
 } ;
 
 #undef offset
@@ -159,6 +163,7 @@ static void WMMessageCloseCallback
 ( Widget gw, caddr_t client, caddr_t caller ) ;
 static void WMShellDestroyCallback
 ( Widget gw, caddr_t client, caddr_t caller ) ;
+static void		SeparateWin_fixcallback		(Widget, XtPointer, XtPointer) ;
 
 static void skkinput_FullDrawModeLine
 ( Widget gw, struct SKKInputNode *node, 
@@ -368,85 +373,76 @@ static void SEPW_CanvasPopupPositionInit
   return ;
 }
 
-static void SEPW_CreateCanvasPopup( Widget gw )
+static void
+SEPW_CreateCanvasPopup (
+	register Widget		gw)
 {
-  SeparateWinWidget w = ( SeparateWinWidget )gw ;
-  unsigned int winwidth, winheight ; 
-  XSizeHints sizehints ;
-  Widget canvas_popup, canvas ;
-  Arg arg[ 10 ] ;
-  Cardinal i ;
+	SeparateWinWidget w = ( SeparateWinWidget )gw ;
+	unsigned int winwidth, winheight ; 
+	XSizeHints sizehints ;
+	Widget canvas_popup, canvas ;
+	Arg arg[ 10 ] ;
+	Cardinal i ;
 
-  winwidth  = w->core.width  ;
-  winheight = ( w->core.height * w->separateWin.font_height ) ;
-  
-  i = 0 ;
-  XtSetArg( arg[ i ], XtNinput, TRUE ) ;
-  i ++ ;
-  XtSetArg( arg[ i ], XtNmappedWhenManaged, TRUE ) ;
-  i ++ ;
-  XtSetArg( arg[ i ], XtNwidth, winwidth ) ;
-  i ++ ;
-  XtSetArg( arg[ i ], XtNheight, winheight ) ;
-  i ++ ;
-  /* ޤݥåץåץ롣*/
-  canvas_popup = XtCreatePopupShell
-    ( "skkinput", wmcloseShellWidgetClass, gw, arg, i ) ;
-  XtAddCallback
-    ( canvas_popup, XtNwmcloseNotify, 
-      ( XtCallbackProc )WMMessageCloseCallback, gw ) ;
-  XtAddCallback
-    ( canvas_popup, XtNdestroyNotify, 
-      ( XtCallbackProc )WMShellDestroyCallback, gw ) ;
-  /* βˤĤɽѤ widget 롣shellwidget ľ *
-   * ܽ񤤤ƤϤޤäȤΤǡġ*/
-  canvas = XtVaCreateManagedWidget
-    ( "canvas", canvasWidgetClass, canvas_popup, NULL ) ;
-  XtAddCallback
-    ( canvas, XtNredrawNotify, 
-      ( XtCallbackProc )SeparateCanvas_RedisplayCallback, gw ) ;
-  XtAddCallback
-    ( canvas, XtNdestroyNotify, 
-      ( XtCallbackProc )Canvas_DestroyCallback, gw ) ;
-  /* realize Ƥ*/
-  w->separateWin.canvas_popup = canvas_popup ;
-  w->separateWin.canvas = canvas ;
-  XtRealizeWidget( canvas_popup ) ;
-  XtRealizeWidget( canvas ) ;
+	winwidth  = w->core.width  ;
+	winheight = ( w->core.height * w->separateWin.font_height ) ;
+	
+	i = 0 ;
+	XtSetArg( arg[ i ], XtNinput, w->separateWin.m_fInputFocus) ;
+	i ++ ;
+	XtSetArg( arg[ i ], XtNmappedWhenManaged, TRUE ) ;
+	i ++ ;
+	XtSetArg( arg[ i ], XtNwidth, winwidth ) ;
+	i ++ ;
+	XtSetArg( arg[ i ], XtNheight, winheight ) ;
+	i ++ ;
+	/* ޤݥåץåץ롣*/
+	canvas_popup = XtCreatePopupShell ("skkinput", wmcloseShellWidgetClass, gw, arg, i) ;
+	XtAddCallback (canvas_popup, XtNwmcloseNotify, (XtCallbackProc) WMMessageCloseCallback, gw) ;
+	XtAddCallback (canvas_popup, XtNdestroyNotify, (XtCallbackProc) WMShellDestroyCallback, gw) ;
+	/* βˤĤɽѤ widget 롣shellwidget ľ *
+	 * ܽ񤤤ƤϤޤäȤΤǡġ*/
+	canvas = XtVaCreateManagedWidget ("canvas", canvasWidgetClass, canvas_popup, NULL) ;
+	XtAddCallback (canvas, XtNredrawNotify,  (XtCallbackProc)SeparateCanvas_RedisplayCallback, gw) ;
+	XtAddCallback (canvas, XtNdestroyNotify, (XtCallbackProc )Canvas_DestroyCallback, gw) ;
+	/* realize Ƥ*/
+	w->separateWin.canvas_popup = canvas_popup ;
+	w->separateWin.canvas = canvas ;
+	XtRealizeWidget( canvas_popup ) ;
+	XtRealizeWidget( canvas ) ;
 
-  canvas_popup->core.border_width = 0 ;
-  /* ηǤ򲿤ȤǤ褦ˤɤȤϻפ衣*/
-  winwidth  += canvas_popup->core.border_width * 2 ;
-  winheight += canvas_popup->core.border_width * 2 ;
+	canvas_popup->core.border_width = 0 ;
+	/* ηǤ򲿤ȤǤ褦ˤɤȤϻפ衣*/
+	winwidth  += canvas_popup->core.border_width * 2 ;
+	winheight += canvas_popup->core.border_width * 2 ;
 
-  /* Ƥ˥ꥵäƤ롩 */
-  (void) XtMakeResizeRequest
-    ( ( Widget )canvas_popup,
-      ( Dimension )winwidth, ( Dimension )winheight,
-      &w->core.width, &w->core.height ) ;
-
-  /* ɥΥꥵ¾ꡣ*/
-  sizehints.win_gravity = NorthWestGravity ;
-  sizehints.base_width  = 0 ; /*canvas_popup->core.border_width * 2 ;*/
-  sizehints.base_height = 0 ; /*canvas_popup->core.border_width * 2 ;*/
-  sizehints.width      = winwidth ;
-  sizehints.height     = winheight ;
-  sizehints.width_inc  = 1 ;
-  sizehints.height_inc = w->separateWin.font_height ;
-  sizehints.min_width  = sizehints.base_width ;
-  sizehints.min_height = w->separateWin.font_height * 3 +
-    sizehints.base_height ;
-  sizehints.flags      = 
-    ( PBaseSize | PMinSize | PResizeInc | USSize | PWinGravity ) ;
-
-  /* Window Manager ˥ҥȤäƤ롣*/
-  XSetWMNormalHints
-    ( XtDisplay( gw ), XtWindow( canvas_popup ), &sizehints ) ;
+	/* Ƥ˥ꥵäƤ롩 */
+	(void) XtMakeResizeRequest
+		( ( Widget )canvas_popup,
+		  ( Dimension )winwidth, ( Dimension )winheight,
+		  &w->core.width, &w->core.height ) ;
+	
+	/* ɥΥꥵ¾ꡣ*/
+	sizehints.win_gravity = NorthWestGravity ;
+	sizehints.base_width  = 0 ; /*canvas_popup->core.border_width * 2 ;*/
+	sizehints.base_height = 0 ; /*canvas_popup->core.border_width * 2 ;*/
+	sizehints.width      = winwidth ;
+	sizehints.height     = winheight ;
+	sizehints.width_inc  = 1 ;
+	sizehints.height_inc = w->separateWin.font_height ;
+	sizehints.min_width  = sizehints.base_width ;
+	sizehints.min_height = w->separateWin.font_height * 3 +
+		sizehints.base_height ;
+	sizehints.flags      = 
+		( PBaseSize | PMinSize | PResizeInc | USSize | PWinGravity ) ;
+	
+	/* Window Manager ˥ҥȤäƤ롣*/
+	XSetWMNormalHints (XtDisplay (gw), XtWindow (canvas_popup), &sizehints) ;
 
   /* ɥɽ֤׻롣*/
-  SEPW_CanvasPopupPositionInit
-    ( gw, w->separateWin.focus_window,
-      winwidth, winheight, canvas_popup->core.border_width ) ;
+	SEPW_CanvasPopupPositionInit
+		( gw, w->separateWin.focus_window,
+		  winwidth, winheight, canvas_popup->core.border_width ) ;
   return ;
 }
 
@@ -459,50 +455,46 @@ static void SEPW_CreateCanvasPopup( Widget gw )
 static void SEPW_Realize
 ( Widget gw, XtValueMask *valueMask, XSetWindowAttributes *attrs )
 {
-  SeparateWinWidget w = ( SeparateWinWidget )gw ;
-  
-  CoreWidgetClass super =
-    ( CoreWidgetClass )XtClass( gw )->core_class.superclass ;
+	SeparateWinWidget	w = ( SeparateWinWidget )gw ;
+	CoreWidgetClass		super = ( CoreWidgetClass )XtClass( gw )->core_class.superclass ;
 
-  /* ŬڤʥɥѰդƤ*/
-  ( *super->core_class.realize )( gw, valueMask, attrs ) ;
+	/* ŬڤʥɥѰդƤ*/
+	( *super->core_class.realize )( gw, valueMask, attrs ) ;
 
-  /* եɥꤵƤʤ꤬롣*/
-  if( w->separateWin.focus_window == None ){
-    if( w->separateWin.client_window == None ){
-      XtDestroyWidget( gw ) ;
-      return ;
-    }
-    w->separateWin.focus_window = w->separateWin.client_window ;
-  }
-  /* եȤְ꤬äƤ顢λ롣(ɽʤ) */
-  if( !fontMgr_GetFontSetInfo
-      ( w->separateWin.fontset,
-	&w->separateWin.font_height,
-	&w->separateWin.font_ascent ) ){
-    XtDestroyWidget( gw ) ;
-    return ;
-  }
-  /* Popup ȤβΥХ롣*/
-  SEPW_CreateCanvasPopup( gw ) ;
-  /* gc 롣оݤȤʤ Window ޤ GC ϺʤȤ
-   * ߥǤ͡ */
-  SEPW_CreateAllGCs( gw ) ;
-
-  /* ɥ򵯤*/
-  XtPopup( w->separateWin.canvas_popup, XtGrabNone ) ;
-  w->separateWin.canvas_probe = True ;
-  /* X Input Method ǤʤСեå*/
-  XtSetKeyboardFocus( w->separateWin.canvas_popup, gw ) ;
-  XtSetKeyboardFocus( w->separateWin.canvas, gw ) ;
-  /* ̤κ*/
-  XtVaSetValues
-    ( w->separateWin.canvas, XtNredrawRequest, True, NULL ) ;
-#if 0
-  SEPW_RedrawScreen( gw ) ;
-  XSync( XtDisplay( gw ), False ) ;
-#endif
-  return ;
+	/* եɥꤵƤʤ꤬롣*/
+	if( w->separateWin.focus_window == None ){
+		if( w->separateWin.client_window == None ){
+			XtDestroyWidget( gw ) ;
+			return ;
+		}
+		w->separateWin.focus_window = w->separateWin.client_window ;
+	}
+	/* եȤְ꤬äƤ顢λ롣(ɽʤ) */
+	if( !fontMgr_GetFontSetInfo
+		( w->separateWin.fontset,
+		  &w->separateWin.font_height,
+		  &w->separateWin.font_ascent ) ){
+		XtDestroyWidget( gw ) ;
+		return ;
+	}
+	if (XtHasCallbacks (gw, XtNfixNotify) != XtCallbackHasSome) 
+		XtAddCallback (gw, XtNfixNotify, SeparateWin_fixcallback, NULL) ;
+	
+	/* Popup ȤβΥХ롣*/
+	SEPW_CreateCanvasPopup (gw) ;
+	SEPW_CreateAllGCs (gw) ;
+	
+	/* ɥ򵯤*/
+	XtPopup (w->separateWin.canvas_popup, XtGrabNone) ;
+	w->separateWin.canvas_probe = True ;
+	/* X Input Method ǤʤСեå*/
+	if (w->separateWin.m_fInputFocus) {
+		XtSetKeyboardFocus (w->separateWin.canvas_popup, gw) ;
+		XtSetKeyboardFocus (w->separateWin.canvas, gw) ;
+	}
+	/* ̤κ*/
+	XtVaSetValues (w->separateWin.canvas, XtNredrawRequest, True, NULL) ;
+	return ;
 }
 
 
@@ -1432,3 +1424,14 @@ static void WMShellDestroyCallback
   XtRemoveAllCallbacks( gw, XtNdestroyNotify ) ;
   return ;
 }
+
+static void
+SeparateWin_fixcallback (
+	register Widget		gw,
+	register XtPointer	client,
+	register XtPointer	caller)
+{
+	XtCallCallbacks (gw, XtNtextNotify, caller) ;
+	return ;
+}
+
